from flask import Blueprint, request, jsonify, make_response
from flask_jwt_extended import create_access_token, create_refresh_token, jwt_required, get_jwt_identity, unset_jwt_cookies
from app.models.user import User
from app.utils.response import success_response, error_response, check_db_connection
from app import db, jwt
import datetime
from datetime import datetime as dtm
from flask_mail import Message
from itsdangerous import URLSafeTimedSerializer
from flasgger import swag_from
import json
import pytz

jakarta_timezone = pytz.timezone('Asia/Jakarta')
current_time = dtm.now(jakarta_timezone)

auth_bp = Blueprint('auth', __name__)

@auth_bp.route('/register', methods=['POST'])
@swag_from({
    'responses': {
        200: {
            'description': 'A user registration response',
            'examples': {
                'application/json': {
                    'code': 200,
                    'message': 'User registered successfully.',
                    'data': {
                        'username': 'string',
                        'email': 'string'
                    }
                }
            }
        }
    }
})
def register():

    connection_ok, connection_error = check_db_connection()
    if not connection_ok:
        return error_response(f"Database connection failed: {connection_error}", code=500)

    data = request.get_json()
    created_date = current_time
    first_name = data.get('first_name')
    last_name = data.get('last_name')
    username = data.get('username')
    password = data.get('password')
    email   = data.get('email')
    role_id = data.get('role')
    
    if User.query.filter_by(username=username).first():
        return error_response(message="User already exists", code=400)

    new_user = User(username=username, email=email, created_date=created_date, first_name=first_name, last_name=last_name, role_id=role_id)
    new_user.set_password(password)
    db.session.add(new_user)
    db.session.commit()
    return success_response(message="User created successfully", code=200)

@auth_bp.route('/login', methods=['POST'])
def login():

    connection_ok, connection_error = check_db_connection()
    if not connection_ok:
        return error_response(f"Database connection failed: {connection_error}", code=500)

    username = request.json.get('username', None)
    password = request.json.get('password', None)
    
    user = User.query.filter_by(username=username).first()
    if user is None or not user.check_password(password):
        return error_response("Bad username or password", code=401)

    # Remove old refresh token if it exists
    old_refresh_token = user.current_refresh_token
    if old_refresh_token:
        # Assuming you are storing tokens in a database or cache, adjust this accordingly
        # Do not try to delete a string with SQLAlchemy
        # Just update the user's refresh token if needed
        user.current_refresh_token = None
        db.session.commit()

    access_token_expires = datetime.timedelta(minutes=request.json.get('access_token_expires', 15))
    refresh_token_expires = datetime.timedelta(days=request.json.get('refresh_token_expires', 7))

    access_token = create_access_token(identity=username, expires_delta=access_token_expires)
    refresh_token = create_refresh_token(identity=username, expires_delta=refresh_token_expires)

    # Update user with the new refresh token
    user.current_refresh_token = refresh_token
    db.session.commit()
    usr = {
            'id': user.id,
            'username': user.username,
            'created_date': user.created_date,
            'updated_date': user.updated_date,
            'role_id': user.role_id,
            'first_name': user.first_name,
            'last_name': user.last_name,
            'email': user.email,
            "access_token": access_token, 
            "refresh_token": refresh_token
        }
    
    return success_response(data=usr)

@auth_bp.route('/logout', methods=['GET'])
@jwt_required(refresh=True)
def logout():
    current_user = get_jwt_identity()  # This should be a string (username)
    user = User.query.filter_by(username=current_user).first()
    if user:
        # Clear the refresh token
        user.current_refresh_token = None
        db.session.commit()

    # Create a response object
    response = make_response(success_response(message="Successfully logged out"))

    # Unset JWT cookies
    unset_jwt_cookies(response)

    return response
