from flask import Blueprint, jsonify, request
from flask_jwt_extended import jwt_required, get_jwt_identity
from app.models.user import User
from app.utils.response import success_response, error_response, check_db_connection
from app import db, jwt
from app.helper.Master import Master

user_bp = Blueprint('user', __name__)

@user_bp.route('/profile', methods=['GET'])
@jwt_required()
def profile():
    connection_ok, connection_error = check_db_connection()
    if not connection_ok:
        return error_response(f"Database connection failed: {connection_error}", code=500)

    current_user = get_jwt_identity()  # This should be a string (username)
    user = User.query.filter_by(username=current_user).first()
    
    if user.current_refresh_token is None:
        return error_response("User is not Loggin", code=403)

    if user:
        if 'password' in Master.row2dict(user):
            del Master.row2dict(user)['password']
        return success_response(data= Master.row2dict(user))

    return error_response("User not found", code=404)

@user_bp.route('/all', methods=['GET'])
@jwt_required()
def get_users():
    connection_ok, connection_error = check_db_connection()
    if not connection_ok:
        return error_response(f"Database connection failed: {connection_error}", code=500)

    users = User.query.all()
    users_data = [Master.row2dict(user) for user in users]
    for usr in users_data:
        if 'password' in usr:
            del usr['password']
        if 'current_refresh_token' in usr:
            del usr['current_refresh_token']

    return success_response(data=users_data)

@user_bp.route('/<int:user_id>', methods=['GET'])
@jwt_required()
def get_user(user_id):
    connection_ok, connection_error = check_db_connection()
    if not connection_ok:
        return error_response(f"Database connection failed: {connection_error}", code=500)

    user = User.query.get_or_404(user_id)
    user_data = {"id": user.id, "username": user.username, "email": user.email}
    return success_response(data=user_data)

@user_bp.route('/create', methods=['POST'])
@jwt_required()
def create_user():
    connection_ok, connection_error = check_db_connection()
    if not connection_ok:
        return error_response(f"Database connection failed: {connection_error}", code=500)

    data = request.json
    username = data.get('username')
    email = data.get('email')
    password = data.get('password')
    role_names = data.get('roles', [])

    if not username or not email or not password:
        return error_response("Username, email, and password are required", code=400)

    user = User(username=username, email=email)
    user.set_password(password)

    for role_name in role_names:
        role = Role.query.filter_by(name=role_name).first()
        if role:
            user.roles.append(role)

    db.session.add(user)
    db.session.commit()
    return success_response(message="User created successfully")

@user_bp.route('/roles', methods=['GET'])
@jwt_required()
def get_roles():
    connection_ok, connection_error = check_db_connection()
    if not connection_ok:
        return error_response(f"Database connection failed: {connection_error}", code=500)

    roles = Role.query.all()
    roles_data = [{"id": role.id, "name": role.name, "privileges": [priv.name for priv in role.privileges]} for role in roles]
    return success_response(data=roles_data)

@user_bp.route('/roles', methods=['POST'])
@jwt_required()
def create_role():
    connection_ok, connection_error = check_db_connection()
    if not connection_ok:
        return error_response(f"Database connection failed: {connection_error}", code=500)

    data = request.json
    role_name = data.get('name')
    privilege_names = data.get('privileges', [])

    if not role_name:
        return error_response("Role name is required", code=400)

    role = Role(name=role_name)

    for priv_name in privilege_names:
        privilege = Privilege.query.filter_by(name=priv_name).first()
        if privilege:
            role.privileges.append(privilege)

    db.session.add(role)
    db.session.commit()
    return success_response(message="Role created successfully")

@user_bp.route('/<int:user_id>', methods=['PUT'])
@jwt_required()
def update_user(user_id):

    connection_ok, connection_error = check_db_connection()
    if not connection_ok:
        return error_response(f"Database connection failed: {connection_error}", code=500)

    user = User.query.get_or_404(user_id)
    data = request.json
    username = data.get('username')
    email = data.get('email')
    password = data.get('password')
    role_names = data.get('roles', [])

    if username:
        user.username = username
    if email:
        user.email = email
    if password:
        user.set_password(password)
    if role_names is not None:
        user.roles = []
        for role_name in role_names:
            role = Role.query.filter_by(name=role_name).first()
            if role:
                user.roles.append(role)

    db.session.commit()
    return success_response(message="User updated successfully")

@user_bp.route('/roles/<int:role_id>', methods=['PUT'])
@jwt_required()
def update_role(role_id):
    connection_ok, connection_error = check_db_connection()
    if not connection_ok:
        return error_response(f"Database connection failed: {connection_error}", code=500)

    role = Role.query.get_or_404(role_id)
    data = request.json
    role_name = data.get('name')
    privilege_names = data.get('privileges', [])

    if role_name:
        role.name = role_name
    if privilege_names is not None:
        role.privileges = []
        for priv_name in privilege_names:
            privilege = Privilege.query.filter_by(name=priv_name).first()
            if privilege:
                role.privileges.append(privilege)

    db.session.commit()
    return success_response(message="Role updated successfully")